/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <Elementary.h>
#include <vconf.h>
#include <Ecore_Input.h>
#include <feedback.h>
#include <tzsh.h>
#include <tzsh_quickpanel_service.h>
#include <notification.h>

#if defined(WINSYS_X11)
#include <Ecore_X.h>
#include <X11/Xlib.h>
#include <utilX.h>
#endif

#include "quickpanel-ui.h" // appdata
#include "common_uic.h"
#include "common.h"
#include "noti_util.h"
#include "keyboard.h"

#define KEY_BACK	"XF86Back"
#define KEY_CANCEL	"Cancel"
#define KEY_MENU	"XF86Menu"
#define KEY_QUICKPANEL	"XF86QuickPanel"
#define KEY_HOME	"XF86Home"

static Eina_Bool _service_hardkey_up_cb(void *data, int type, void *event)
{
	struct appdata *ad = NULL;
	Ecore_Event_Key *key_event = NULL;

	retif(data == NULL || event == NULL, EINA_FALSE, "Invalid parameter!");
	ad = data;
	key_event = event;

	if (!strcmp(key_event->keyname, KEY_HOME)) {
		if (ad->is_hardkey_cancel == EINA_FALSE) {
			quickpanel_uic_close_quickpanel(false, 0);
		} else {
			DBG("Cancel status, do nothing");
		}
	} else if (!strcmp(key_event->keyname, KEY_CANCEL)) {
		ad->is_hardkey_cancel = EINA_FALSE;
	} else if (!strcmp(key_event->keyname, KEY_BACK)) {
		if (ad->popup != NULL) {
			Evas_Smart_Cb back_cb = evas_object_data_get(ad->popup, EDATA_BACKKEY_CB);
			if (back_cb != NULL) {
				back_cb(ad->popup, ad->popup, NULL);
			}
		} else {
			quickpanel_uic_close_quickpanel(false, 0);
		}
	}
	return EINA_FALSE;
}

static Eina_Bool _service_hardkey_down_cb(void *data, int type, void *event)
{
	Ecore_Event_Key *key_event = event;
	struct appdata *ad = data;
	retif(key_event == NULL, EINA_FALSE, "Invalid parameter!");
	retif(ad == NULL, EINA_FALSE, "Invalid parameter!");

	if (!strcmp(key_event->keyname, KEY_CANCEL)) {
		ad->is_hardkey_cancel = EINA_TRUE;
	} else if (!strcmp(key_event->keyname, KEY_QUICKPANEL)) {
		quickpanel_uic_toggle_openning_quickpanel();
	}
	return EINA_FALSE;
}

HAPI void quickpanel_keyboard_init(void *data)
{
	struct appdata *ad = data;
	Ecore_Event_Handler *hdl_key_down = NULL;
	Ecore_Event_Handler *hdl_key_up = NULL;
	retif(ad == NULL, , "Invalid parameter!");

#if defined(WINSYS_X11)
	if (utilx_grab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_QUICKPANEL, SHARED_GRAB) != 0) {
		ERR("failed to grab KEY_QUICKPANEL");
	}
#endif

	hdl_key_down = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _service_hardkey_down_cb, ad);
	if (hdl_key_down == NULL) {
		ERR("failed to add handler(ECORE_EVENT_KEY_DOWN)");
	}
	ad->hdl_hardkey_down = hdl_key_down;

	hdl_key_up = ecore_event_handler_add(ECORE_EVENT_KEY_UP, _service_hardkey_up_cb, ad);
	if (hdl_key_up == NULL) {
		ERR("failed to add handler(ECORE_EVENT_KEY_UP)");
	}
	ad->hdl_hardkey_up = hdl_key_up;
	ad->is_hardkey_cancel = EINA_FALSE;
}

HAPI void quickpanel_keyboard_fini(void *data)
{
	struct appdata *ad = data;
	retif(ad == NULL, , "Invalid parameter!");

	if (ad->hdl_hardkey_up != NULL) {
		ecore_event_handler_del(ad->hdl_hardkey_up);
		ad->hdl_hardkey_up = NULL;
	}

	if (ad->hdl_hardkey_down != NULL) {
		ecore_event_handler_del(ad->hdl_hardkey_down);
		ad->hdl_hardkey_down = NULL;
	}

#if defined(WINSYS_X11)
	if (utilx_ungrab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_QUICKPANEL) != 0) {
		ERR("failed to ungrab KEY_QUICKPANEL");
	}
#endif
}

HAPI void quickpanel_keyboard_openning_init(void *data)
{
	struct appdata *ad = data;
	retif(ad == NULL, , "Invalid parameter!");

#if defined(WINSYS_X11)
	if (utilx_grab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_BACK, EXCLUSIVE_GRAB) != 0) {
		ERR("failed to grab KEY_BACK");
	}

	if (utilx_grab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_MENU, EXCLUSIVE_GRAB) != 0) {
		ERR("failed to grab KEY_MENU");
	}

	if (utilx_grab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_HOME, SHARED_GRAB) != 0) {
		ERR("failed to grab KEY_HOME");
	}
#endif
}

HAPI void quickpanel_keyboard_closing_fini(void *data)
{
	struct appdata *ad = data;
	retif(ad == NULL, , "Invalid parameter!");

#if defined(WINSYS_X11)
	if (utilx_ungrab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_BACK) != 0) {
		ERR("failed to ungrab KEY_BACK");
	}

	if (utilx_ungrab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_MENU) != 0) {
		ERR("failed to ungrab KEY_MENU");
	}

	if (utilx_ungrab_key(ecore_x_display_get(), elm_win_xwindow_get(ad->win), KEY_HOME) != 0) {
		ERR("failed to ungrab KEY_HOME");
	}
#endif
}
